"""
Convert to and from Roman numerals
This program is part of 'Dive Into Python 3', a free Python book for
experienced programmers. Visit http://diveintopython3.org/ for the latest
version.

https://github.com/kennethreitz/dive-into-python3/blob/master/examples/roman7.py
"""


import re


roman_numeral_map = (
    ("M", 1000),
    ("CM", 900),
    ("D", 500),
    ("CD", 400),
    ("C", 100),
    ("XC", 90),
    ("L", 50),
    ("XL", 40),
    ("X", 10),
    ("IX", 9),
    ("V", 5),
    ("IV", 4),
    ("I", 1),
)

arabic_pattern = re.compile(r"(\d+)")

roman_pattern = re.compile(
    """
    M{0,3}              # thousands - 0 to 3 M's
    (CM|CD|D?C{0,3})    # hundreds - 900 (CM), 400 (CD), 0-300 (0 to 3 C's),
                        #            or 500-800 (D, followed by 0 to 3 C's)
    (XC|XL|L?X{0,3})    # tens - 90 (XC), 40 (XL), 0-30 (0 to 3 X's),
                        #        or 50-80 (L, followed by 0 to 3 X's)
    (IX|IV|V?I{0,3})    # ones - 9 (IX), 4 (IV), 0-3 (0 to 3 I's),
                        #        or 5-8 (V, followed by 0 to 3 I's)
    """,
    re.VERBOSE,
)


def roman(n):
    """Convert integer to Roman numeral"""
    try:
        n = int(n)
    except ValueError:
        return n
    if not (0 < n < 4000):
        return n
    result = ""
    for numeral, integer in roman_numeral_map:
        while n >= integer:
            result += numeral
            n -= integer
    return result


def arabic(s):
    """Convert Roman numeral to integer"""
    if not s:
        return s
    result = 0
    index = 0
    for numeral, integer in roman_numeral_map:
        while s[index : index + len(numeral)] == numeral:
            result += integer
            index += len(numeral)
    return result



from django.template import Context, Template
from django.test import TestCase


class RomanTestCase(TestCase):
    """Roman test cases."""

    def test_roman_text(self):
        out = Template("{% load roman_tags %}" "{{ '1999'|roman }}").render(Context())
        self.assertEqual(out, '<span class="numerals numerals-roman">MCMXCIX</span>')

    def test_roman_integer(self):
        out = Template("{% load roman_tags %}" "{{ 1999|roman }}").render(Context())
        self.assertEqual(out, '<span class="numerals numerals-roman">MCMXCIX</span>')

    def test_roman_text_context(self):
        out = Template("{% load roman_tags %}" "{{ year|roman }}").render(
            Context(
                {
                    "year": "1999",
                }
            )
        )
        self.assertEqual(out, '<span class="numerals numerals-roman">MCMXCIX</span>')

    def test_roman_integer_context(self):
        out = Template("{% load roman_tags %}" "{{ year|roman }}").render(
            Context(
                {
                    "year": 1999,
                }
            )
        )
        self.assertEqual(out, '<span class="numerals numerals-roman">MCMXCIX</span>')

    def test_roman_words(self):
        out = Template(
            "{% load roman_tags %}"
            "{{ 'Party like it is 1999. Or maybe 2001?'|roman }}"
        ).render(Context())
        self.assertEqual(
            out,
            'Party like it is <span class="numerals numerals-roman">MCMXCIX</span>. Or maybe <span class="numerals numerals-roman">MMI</span>?',
        )

    def test_roman_words_context(self):
        out = Template("{% load roman_tags %}" "{{ words|roman }}").render(
            Context({"words": "Party like it is 1999. Or maybe 2001?"})
        )
        self.assertEqual(
            out,
            'Party like it is <span class="numerals numerals-roman">MCMXCIX</span>. Or maybe <span class="numerals numerals-roman">MMI</span>?',
        )

    def test_roman_now(self):
        out = Template(
            "{% load roman_tags %}" "{% filter roman %}{% now 'Y' %}{% endfilter %}."
        ).render(Context())
        self.assertEqual(out, '<span class="numerals numerals-roman">MMXXII</span>.')

    def test_arabic_text(self):
        out = Template("{% load roman_tags %}" "{{ 'MCMXCIX'|arabic }}").render(
            Context()
        )
        self.assertEqual(out, '<span class="numerals numerals-arabic">1999</span>')

    def test_arabic_text_context(self):
        out = Template("{% load roman_tags %}" "{{ year|arabic }}").render(
            Context(
                {
                    "year": "MCMXCIX",
                }
            )
        )
        self.assertEqual(out, '<span class="numerals numerals-arabic">1999</span>')

    def test_arabic_words(self):
        out = Template(
            "{% load roman_tags %}"
            "{{ 'Party like it is MCMXCIX. Or maybe MMI?'|arabic }}"
        ).render(Context())
        self.assertEqual(
            out,
            'Party like it is <span class="numerals numerals-arabic">1999</span>. Or maybe <span class="numerals numerals-arabic">2001</span>?',
        )

    def test_arabic_words_context(self):
        out = Template("{% load roman_tags %}" "{{ words|arabic }}").render(
            Context({"words": "Party like it is MCMXCIX. Or maybe MMI?"})
        )
        self.assertEqual(
            out,
            'Party like it is <span class="numerals numerals-arabic">1999</span>. Or maybe <span class="numerals numerals-arabic">2001</span>?',
        )

    def test_arabic_richard(self):
        out = Template(
            "{% load roman_tags %}"
            "{{ 'Where did the princes go, Richard III of England?'|arabic }}"
        ).render(Context())
        self.assertEqual(
            out,
            'Where did the princes go, Richard <span class="numerals numerals-arabic">3</span> of England?',
        )



import re

from django import template
from django.template.defaultfilters import stringfilter
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe

from ..roman import (
    arabic as _arabic,
    arabic_pattern,
    roman as _roman,
    roman_pattern,
)

register = template.Library()


def roman_repl(match_obj):
    if match_obj and match_obj.group():
        html = '<span class="numerals numerals-roman">%s</span>'
        return html % _roman(match_obj.group())
    return ""


def arabic_repl(match_obj):
    if match_obj and match_obj.group():
        html = '<span class="numerals numerals-arabic">%s</span>'
        return html % _arabic(match_obj.group())
    return ""


@register.filter(name="roman", is_safe=True, needs_autoescape=True)
@stringfilter
def roman_filter(value, autoescape=True):
    if autoescape:
        value = conditional_escape(value)
    return mark_safe(arabic_pattern.sub(roman_repl, value))


@register.filter(name="arabic", is_safe=True, needs_autoescape=True)
@stringfilter
def arabic_filter(value, autoescape=True):
    if autoescape:
        value = conditional_escape(value)
    return mark_safe(roman_pattern.sub(arabic_repl, value))



import os

from setuptools import find_packages, setup


setup(
    name="django-roman",
    version="0.2.3",
    description="A Django template filter to convert Arabic numerals to Roman numerals",
    long_description=open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.rst")
    ).read(),
    long_description_content_type="text/x-rst",
    author="Richard Cornish",
    author_email="rich@richardcornish.com",
    url="https://github.com/richardcornish/django-roman",
    license="BSD",
    packages=find_packages(exclude=("tests",)),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    python_requires=">=3.6",
    install_requires=["Django>=2.2"],
)



import os


BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = "fake-key"

INSTALLED_APPS = [
    "roman",
]

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
        "TEST": {
            "NAME": None,
        },
    }
}

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
    }
]
